#include "traffic_light_detect.h"

TrafficLightDetect::TrafficLightDetect(uint8_t port) : MeSerial(port)
{

}

void TrafficLightDetect::loop()
{
  read_serial_port();
}

boolean TrafficLightDetect::set_color_range(uint8_t color,uint8_t type,uint8_t h,uint8_t s,uint8_t v)
{
  printf("G11 C%d T%d H%d S%d V%d\r\n",color,type,h,s,v);
  delay(50);
}

boolean TrafficLightDetect::set_hough_circles_parameter(uint8_t color,int16_t minDist,int16_t operator_threshold,int16_t accumulator_threshold,int16_t min_radius,int16_t max_radius)
{
  printf("G12 C%d D%d O%d A%d I%d X%d\r\n",color,minDist,operator_threshold,accumulator_threshold,min_radius,max_radius);
  delay(50);
}

boolean TrafficLightDetect::set_hs_judgment_threshold(uint8_t color,int16_t threshold)
{
  printf("G13 C%d T%d\r\n",color,threshold);
  delay(50);
}

uint8_t TrafficLightDetect::get_traffic_light_status(void)
{
  return traffic_light_status;
}

boolean TrafficLightDetect::read_serial_port(void)
{
  if(available() > 0)
  {
    char c = read();
    if((c == 'G') || (c == 'g'))
    {
      bufindex = 0;
      buf[bufindex]=c;
      start_cmd = true;
      bufindex++;
    }
    else if(start_cmd == true)
    {
      buf[bufindex++]=c;
    }

    if(bufindex > 31)
    {
      memset(buf,0,32);
      bufindex = 0;
      start_cmd = false;
    }
    else if((start_cmd == true) && ((c=='\n') || (c=='#')))
    {
      parseCmd(buf);
      memset(buf,0,32);
      bufindex = 0;
      start_cmd = false;
      return data_flag;
    }
  }
}

//这个函数用于处理一串数据是否符合 G代码的协议
void TrafficLightDetect::parseCmd(char * cmd)
{
  if((cmd[0]=='g') || (cmd[0]=='G'))
  { 
    // gcode
    data_flag = false;
    parseGcode(cmd+1);
  }
}

//这个函数用于处理G代码的内容
void TrafficLightDetect::parseGcode(char *cmd)
{
  char * tmp;
  char * str = "          ";
  char g_code_cmd;
  g_code_cmd = atoi((const char *)cmd);;
  str = strtok_r(cmd, " ", &tmp);
  switch(g_code_cmd)
  {
    case 11:
      {
        int16_t result = 0;
        while(str != NULL)
        {
          str = strtok_r(0, " ", &tmp);
          if((str[0]=='R') || (str[0]=='r'))
          {
            result = atoi(str+1);
          }
        }
        //打印我们读到的命令和值
      }
      break;
    case 20:
      {
        int16_t traffic_light_value = 0;
        while(str != NULL)
        {
          str = strtok_r(0, " ", &tmp);
          if((str[0]=='L') || (str[0]=='l'))
          {
            traffic_light_value = atoi(str+1);
          }
        }
        traffic_light_status = traffic_light_value;
        //打印我们读到的命令和值
      }
      break;
    default:
      break;
  }
}
